#!/bin/sh

###########################################################################
#                                                                         #
#                                                                         #
#              Copyright (c) 2015 ISKRATEL                                #
#                                                                         #
#   Name        : sysup-disable.sh                                        #
#                                                                         #
#   Decription  : Script controls sysup actions disable file.             #
#                                                                         #
#   Position    : rte/etc                                                 #
#                                                                         #
#   Date        : Jan 2015                                                #
#                                                                         #
#   Author      : Klemen Pogacnik                                         #
#                                                                         #
#   Remarks     : * None                                                  #
#                                                                         #
#   Revisions   : * None                                                  #
#                                                                         #
###########################################################################
#INSTRUCTIONS:
#The script is called when some actions with the sysup actions disabled file (SAD file) want to be performed.
#
#Usual procedures:
#---------------------------------------------------------------------------
#   1.SYSUP actions will be disabled for some time on local node
#       Action:  sysup-disable.sh disable "Name" 0
#       Parameters: 
#           Name:  any string which identifies script's caller
#           SessionId=0: This is the first action of a session, so DUMMY session id is entered
#        or SessionId=SINGLE_ENTRY: User is allowed to have only one entry in SAD, so previous records will be removed, if exist
#       Output: 
#           Result: 0 - success /  1 - error on local node
#           SessionId: selected session id is printed to stdout
#
#       Explanation: SAD file is created and a record with name, session id (which is selected by script) 
#                    and time is inserted to that file.
#
#       Example:
#           NewSessionId=`sysup-disable.sh disable "Name" 0` && echo "OK SessionId=$NewSessionId" || echo "ERROR"
#
#---------------------------------------------------------------------------
#   2.SYSUP actions will be disabled for some time on both nodes
#
#       Action:  sysup-disable.sh disable-hsb "Name" 0
#       Parameters: 
#           Name: any string which identifies script's caller
#	    SessionId=0: This is the first action of a session, so DUMMY session id is entered
#        or SessionId=SINGLE_ENTRY: User is allowed to have only one entry in SAD, so previous records will be removed, if exist
#       Output: 
#           Result: 0 - success / 1 - error on local node /  2 - error on remote node
#           SessionId: selected session id is printed to stdout
#
#       Explanation: SAD file is created on both sides and a record with name, session id (which is selected by script) 
#                    and time is inserted to both files.
#
#       Example:
#           NewSessionId=`sysup-disable.sh disable-hsb "Name" 0` && echo "OK SessionId=$NewSessionId" || echo "ERROR"
#
#---------------------------------------------------------------------------
#   3.SYSUP actions will be enabled again on local node
#       Action:  sysup-disable.sh enable "Name" SessionId
#       Parameters:
#           Name: any string which identifies script's caller
#           SessionId: Id which we got on previous script's call with disable parameter (see point 1)
#        or SessionId=USER_ALL: All records of the user with the "Name" will be removed from local SAD file
#        or SessionId=ALL: SOZ file will be removed on local node, regardless how many records are in this file

#       Output: 
#           Result: 0 - success /  1 - error on local node
#           SessionId: entered session id is printed to stdout
#
#       Explanation: A record with session id is removed from SAD file. If this was the last record, whole file is removed.
#
#       Example:
#           NewSessionId=`sysup-disable.sh enable "Name" SessionId` && echo "OK" || echo "ERROR"
#
#---------------------------------------------------------------------------
#   4.SYSUP actions will be enabled again on both nodes
#
#       Action:  sysup-disable.sh enable-hsb "Name" SessionId
#       Parameters: 
#           Name:  any string which identifies script's caller
#	    SessionId: Id which we got on previous script's call with disable-hsb parameter (see point 2)
#        or SessionId=USER_ALL: All records of the user with the "Name" will be removed from local and remote SAD file
#        or SessionId=ALL: SOZ file will be removed on local and remote node, regardless how many records are in this file
#       Output: 
#           Result: 0 - success / 1 - error on local node /  2 - error on remote node
#           SessionId: entered session id is printed to stdout
#
#       Explanation: A record with session id is removed from SAD files on both sides. If this was the last record, 
#                    whole file is removed.
#
#       Example:
#           NewSessionId=`sysup-disable.sh enable-hsb "Name" SessionId` && echo "OK" || echo "ERROR"
#
#---------------------------------------------------------------------------
#   5.Checks if some records are in SOZ file too long (probably somebody forgets to delete them)
#
#       Action:  sysup-disable.sh check
#       Output: 
#           Result: 0 - success / 1 - error on local node /  2 - error on remote node
#           NrOfRecords: number of removed records
#
#       Explanation: Script check each record in local SOZ file and removes those which exceeds
#                    time defined by constant RECORD_MAX_TIMEOUT (see below). If all records are
#                    removed also SOZ file is removed.
#
#       Example:
#           NrOfRec=`sysup-disable.sh check` && echo "OK - Nr. of records removed=$NrOfRec" || echo "ERROR"
#	   
#---------------------------------------------------------------------------
#   6.Checks if sysup actions are enabled or disabled (SOZ file is created or not)
#
#       Action:  sysup-disable.sh status
#       Output: 
#           Result: 0 - success / 1 - error on local node /  2 - error on remote node
#           Status: ENABLED / DISABLED with NrOfRecords
#
#       Explanation: Script check SOZ file and print status. If SOZ file doesn't exist ENABLED status is printed.
#                    If SOZ file exists, DISABLED status is printed with number of records in SOZ file ond file's name.
#
#       Example:
#           sysup-disable.sh status
#	   
#---------------------------------------------------------------------------



##################################################
# CONSTANTS
##################################################
readonly RECORD_MAX_TIMEOUT="3700" #max time when record is in the file (in seconds) 
readonly LOCK_TIMEOUT="5"   #lock timeout in sec
readonly SSH_TIMEOUT="4"    #lock timeout in sec
readonly DSY_OK=0           #return result OK
readonly DSY_ERR_LOC=1      #exit code, when error happens on local node
readonly DSY_ERR_REM=2      #exit code, when error happens on remote node
readonly SESSION_ID_DUMMY=0 #dummy session id
readonly FDLOCK="275"       #file descriptor of lock file
readonly USER="root"        #user for ssh
readonly CLISH_SCRIPT="/sbin/cfg-getparams.sh"  #clish script to read remote IP
readonly SYSUP_FILE="/var/lock/sysup/sysupActionsDisabled"  #sysup file
readonly LOG_FILE="/dev/null" #internal log file
readonly HAMGEN_CF="/opt/iskratel/etc/ham.d/hamsysgen.cf"  #second configuration file of HAM where remote IP can be read 
readonly HAM_CF="/opt/iskratel/etc/ham.d/ham.cf"  #configuration file of HAM where remote IP can be read 

#for testing
#readonly USER="kpog"        #user for ssh
#readonly LOG_FILE="/tmp/dsy.log" #internal log file
#readonly HAM_CF="./ham.cf"  #configuration file of HAM where remote IP can be read 
#readonly SYSUP_FILE="/tmp/SysupDisable"  #sysup file

##################################################
# FUNCTIONS
##################################################


##################################################
# Logs message to stdout and LOGGER.
#
# @param LogString  string to log
# @return none
##################################################
function dsyLog
{
    LOGGER="logger -p user.alert -t sysup-disable --"
    MyTime=` date +"%H:%M:%S/%d-%m-%y" `

    echo "${MyTime}: [$$] ${*}" >> ${LOG_FILE}
    echo "[$$] ${*}" | ${LOGGER}
    return
}

##################################################
# Function is called in case of error. It prepares
# variables which will be printed before exiting.
#
# @param LineNr     line nr of function call
# @param ExitCode   code to return when script exits
# @param LogString  string to add to a log message
# @return none
##################################################
function dsyLogError
{
    LineNr=$1
    ExitCode=$2
    LogString=$3

    dsyLog "${LogString} (LineNr=$((LineNr-1)))"

    return
}


##################################################
# Prints usage instructions.
#
# @param none
# @return none
##################################################
function dsyUsage
{
  echo "    USAGE: sysup-disable.sh help|check|status|disable|disable-hsb|enable|enable-hsb [Name] [SessionId]"
  echo
  echo "        help:           show this text"
  echo "        check:          check if some records are too old and remove them"
  echo "        status:         return the current sysup state on local side"
  echo "        disable:        disable sysup actions on local side"
  echo "        disable-hsb:    disable sysup actions on both sides"
  echo "        enable:         enable sysup actions on local side"
  echo "        enable-hsb:     enable sysup actions on both sides"
  echo "                        Additional params for disable/disable-hsb/enable/enable-hsb:"
  echo "                            Name:       name of the script's caller"
  echo "                            SessionId:  session id, which was returned by previous script's call"
  echo "                                        is 0 if this is the first call of a session (disable or disable-hsb)"
  echo "                                        is ALL when file is removed and sysup actions enabled (enable or enable-hsb)"

  return
}


##################################################
# Make lock.
#
# @param ScriptName name of executing script
# @return Result    OK/ERROR                     
##################################################
function dsyLock
{
    Script=$1
    LockFile="/tmp/${Script}.LCK"

    if command -v "flock" >/dev/null 2>&1; then #flock command exists
        eval "exec ${FDLOCK}>${LockFile}" #make file
        chmod 777 ${LockFile} 2>/dev/null
        if flock -x -w ${LOCK_TIMEOUT} ${FDLOCK}; then
            return $DSY_OK
        else 
            return $DSY_ERR_LOC
        fi
    else #flock commant doesn't exist, make locking with locking directory
        ii=0
        while ! mkdir "$LockFile" 2>/dev/null; #try to make directory
        do
            if [ "$ii" -ge "$LOCK_TIMEOUT" ]; then #lock doesn't succeed
                return $DSY_ERR_LOC
            fi
            ii=$((ii+1))
            sleep 1
        done
        #lock secceeded
        trap 'rm -rf "$LockFile"' 0    # remove directory when script finishes
        return $DSY_OK
    fi
}


##################################################
# Chooses new sesion id.
#
# @return SessionId  new session id
# @return Result    OK/ERROR                     
##################################################
function dsyNewSessionId
{
    SessionId=0
    while [ $((16#$SessionId)) -eq 0 ] || [ $? -eq 0 ]
    do
        SessionId=` printf '%04x%04x' $RANDOM $RANDOM ` || #calculate new session ID \
            { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in printf" ; return $DSY_ERR_LOC; } #error 
        SearchString="[ 	]$SessionId[ 	]"
        grep "$SearchString" $SYSUP_FILE 2>/dev/null >/dev/null #check if SessionId already exists
    done

    echo $SessionId

    return $DSY_OK
}


##################################################
# Gets remote server's IP.
#
# @return ServersIp  new session id
# @return Result    OK/ERROR                     
##################################################
function dsyGetRemIp
{
    #read IP
    if command -v "${CLISH_SCRIPT}" >/dev/null 2>&1; then #clish command exists
        RemSrvIp=` ${CLISH_SCRIPT} hsbRemIntIpAddress | sed -e "s/\/.*$//" `
        RemSrvIpScript=${CLISH_SCRIPT}
    else #no clish, IP should be read from cfg file
        if [ -e "$HAMGEN_CF" ]; then
            HamCf="$HAMGEN_CF"
        elif  [ -e "$HAM_CF" ]; then
            HamCf="$HAM_CF"
        else
            dsyLog "ERROR: can't read REM IP (no CLISH and no ham configuration files)"
            echo "0"
            return $DSY_ERR_REM
        fi
        RemSrvIp=` grep "^[ 	]*remip[ 	]*=" ${HamCf} | sed -e "s/^[^1-9]*//" `
        RemSrvIpScript="grep"
    fi

    #check results
    if echo "$RemSrvIp" | grep -E '^[	]*[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}[ 	]*$' 2>/dev/null >/dev/null; then
        #IP is in the right format
        echo ${RemSrvIp}
        return $DSY_OK
    else #wrong format
        dsyLog "ERROR: Script ${RemSrvIpScript} returned wrong IP ${RemSrvIp}"}
        echo "0"
        return $DSY_ERR_REM
    fi
}


##################################################
# Gets system architecture type.
#
# @return ArchType system architecture type (STDAL, HSB)
# @return Result    OK/ERROR                     
##################################################
function dsyGetSysArch
{
    #read IP
    if command -v "${CLISH_SCRIPT}" >/dev/null 2>&1; then #clish command exists
        SysArch=` ${CLISH_SCRIPT} systemType | sed -e "s/\/.*$//" `
        if [ $SysArch = "standalone" ]; then
            SysArchReturn="STDAL"
        elif [ $SysArch = "duplicated" ]; then
            SysArchReturn="HSB"
        elif [ $SysArch = "geo-dual" ]; then
            SysArchReturn="HSB"
        else
            dsyLog "ERROR: CLISH script returned wrong architecture type"
            echo "STDAL"
            return $DSY_ERR_LOC
        fi

    else #no clish, IP should be read from cfg file
        if [ -e "$HAMGEN_CF" ]; then
            HamCf="$HAMGEN_CF"
        elif  [ -e "$HAM_CF" ]; then
            HamCf="$HAM_CF"
        else
            dsyLog "ERROR: can't read architecture type (no CLISH and no ham configuration files)"
            echo "STDAL"
            return $DSY_ERR_LOC
        fi
        SysArch=` grep "^[ 	]*HSB[ 	]*=" ${HamCf} | sed -e "s/^[ 	]*HSB[ 	]*=[ 	]*//" -e "s/[ 	].*$//" `
        if [ $SysArch = "TRUE" ]; then
            SysArchReturn="HSB"
        elif [ $SysArch = "FALSE" ]; then
            SysArchReturn="STDAL"
        else
            dsyLog "ERROR: wrong architecture type read from ham config file"
            echo "STDAL"
            return $DSY_ERR_LOC
        fi
    fi

    #successfully read architecture type
    echo ${SysArchReturn}
    return $DSY_OK
}


##################################################
# Prepares sysup file with a new line.
#
# @param Name       name of caller of the script
# @param SessionId  current session id
# @return Result    OK/ERROR                     
##################################################
function dsyPrepareNewFileDisable
{
    Name=$1
    OldSessionId=$2
    NewSessionId=$3

    MyTime=` date +"%s	%H:%M:%S/%d-%m-%y" ` || \
        { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in date" ; return $DSY_ERR_LOC; } #error 

    SearchString="[ 	]$NewSessionId[ 	]"
    grep "$SearchString" $SYSUP_FILE 2>/dev/null >/dev/null
    if [ $? -eq $DSY_OK ] ; then 
        # SessionId already exists in file, change time
        SearchString="^[ 	]*$Name[ 	][ 	]*$NewSessionId[ 	]"
        grep "$SearchString" $SYSUP_FILE > /dev/null
        if [ $? -ne $DSY_OK ] ; then # SessionId exists but with different name - error
            dsyLogError $LINENO $DSY_ERR_LOC "ERROR: SessionId exists but with different name"
            return $DSY_ERR_LOC
        fi
        sed -e "/$SearchString/ d" < $SYSUP_FILE > ${SYSUP_FILE}.tmp ||  #remove line in a file \
            { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in sed" ; return $DSY_ERR_LOC; } #error 
        echo -e "$Name	$NewSessionId	$MyTime" >> ${SYSUP_FILE}.tmp || \
            { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in echo>>" ; return $DSY_ERR_LOC; } #error 
    else
        # new entry 
        echo -e "$Name	$NewSessionId	$MyTime" > ${SYSUP_FILE}.tmp1 || \
            { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in echo>" ; return $DSY_ERR_LOC; } #error 

        if [ -e "$SYSUP_FILE" ]; then #file exists
            if [ $OldSessionId = "SINGLE_ENTRY" ]; then #only one entry per user allowed
                # remove previous entry from the same user, if exists
                SearchString="^$Name[ 	]"
                sed -e "/$SearchString/ d" < $SYSUP_FILE > ${SYSUP_FILE}.tmp2 ||  #remove line in a file \
                { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in sed" ; return $DSY_ERR_LOC; } #error 
                cat ${SYSUP_FILE}.tmp2 ${SYSUP_FILE}.tmp1 > ${SYSUP_FILE}.tmp || \
                    { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in cat" ; return $DSY_ERR_LOC; } #error 
            else
                cat ${SYSUP_FILE} ${SYSUP_FILE}.tmp1 > ${SYSUP_FILE}.tmp || \
                    { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in cat" ; return $DSY_ERR_LOC; } #error 
            fi 
        else
            mv ${SYSUP_FILE}.tmp1 ${SYSUP_FILE}.tmp || \
                { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in mv" ; return $DSY_ERR_LOC; } #error 
        fi
    fi
    return $DSY_OK
}


##################################################
# Remove line with SessionId and put it to tmp file.
#
# @param Name       name of caller of the script
# @param SessionId  current session id
# @return Result    OK/ERROR                     
##################################################
function dsyPrepareNewFileEnable
{
    Name=$1
    SessionId=$2

    if [ ! -e ${SYSUP_FILE} ]; then #check if file exists
        >${SYSUP_FILE}.tmp
        return $DSY_OK
    fi

    if [ $SessionId = "USER_ALL" ]; then #remove all user's entry
        SearchString="^$Name[ 	]"
        sed -e "/$SearchString/ d" < ${SYSUP_FILE} > ${SYSUP_FILE}.tmp ||  #remove lines in a file \
            { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in sed" ; return $DSY_ERR_LOC; } #error 
    else
        # check if SessionId exists in file
        SearchString="[ 	]$SessionId[ 	]"
        grep "$SearchString" $SYSUP_FILE > /dev/null
        if [ $? -ne $DSY_OK ] ; then  # doesnt exist - error
            dsyLogError $LINENO $DSY_ERR_LOC "ERROR: SessionId=$SessionId doesn't exist in file" 
            return $DSY_ERR_LOC
        fi

        sed -e "/$SearchString/ d" < ${SYSUP_FILE} > ${SYSUP_FILE}.tmp ||  #remove line in a file \
            { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in sed" ; return $DSY_ERR_LOC; } #error 

        SearchString="^[ 	]*$Name[ 	][ 	]*$SessionId[ 	]"
        grep "$SearchString" $SYSUP_FILE > /dev/null || \
            dsyLog "WARNING: SessionId exists in a file, but with different name - still removed"
    fi

    return $DSY_OK
}


##################################################
# Disables sysup actions on local node.
#
# @param Name         name of the caller of the script
# @param SessionId    session id (SESSION_ID_DUMMY if unknown)
# @return SessionId   session id of current session
# @return Result      OK/ERROR                     
##################################################
function dsyDisableLoc
{
    Name=$1
    SessionId=$2

    if [ $SessionId = "SINGLE_ENTRY" ] || [ $((16#$SessionId)) -eq "$SESSION_ID_DUMMY" ]; then
        # no session id, choose new
        NewSessionId=$(dsyNewSessionId) || \
            return $DSY_ERR_LOC #error 
    else
        NewSessionId=$SessionId
    fi

    #prepare new sysup file
    dsyPrepareNewFileDisable $Name $SessionId $NewSessionId|| \
        return $DSY_ERR_LOC #error 

    #disable sysup actions on loc side
    mv ${SYSUP_FILE}.tmp $SYSUP_FILE || \
        { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in mv" ; return $DSY_ERR_LOC; } #error 

    echo $NewSessionId

    return $DSY_OK
}


##################################################
# Disables sysup actions on remote node.
#
# @param Name         name of the caller of the script
# @param SessionId    session id (SESSION_ID_DUMMY if unknown)
# @return Result      OK/ERROR                     
##################################################
function dsyDisableRem
{
    Name=$1
    SessionId=$2

    if [ $((16#$SessionId)) -eq "$SESSION_ID_DUMMY" ] ; then # no session id, error 
        dsyLogError $LINENO $DSY_ERR_REM "ERROR: Dummy Session ID" 
        return $DSY_ERR_REM 
    fi

    #call script on remote side
    RemSrv=$(dsyGetRemIp) || return $DSY_ERR_REM #read IP and return ERROR if not successfull
    SessionIdNew=`ssh -o ConnectTimeout=$SSH_TIMEOUT ${USER}@${RemSrv} "${MyScriptPath} disable $Name $SessionId" 2>/dev/null` || \
        { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Can not connect to remote server $RemSrv" ; return $DSY_ERR_REM; } #error 

    if [ "$SessionId" != "$SessionIdNew" ]; then #error
        dsyLogError $LINENO $DSY_ERR_REM "ERROR: Wrong SessionId got from remote" 
        return $DSY_ERR_REM
    fi

    return $DSY_OK
}


##################################################
# Disables sysup actions on both nodes.
#
# @param Name         name of the caller of the script
# @param SessionId    session id (SESSION_ID_DUMMY if unknown)
# @return SessionId   session id of current session
# @return Result      OK/ERROR                     
##################################################
function dsyDisableHsb
{
    Name=$1
    SessionId=$2

    if [ $SessionId = "SINGLE_ENTRY" ]; then
        # remove previous records of the same user on remote side, if exists
        dsyEnableRem $Name "USER_ALL" || \
            return $DSY_ERR_REM #error 
        SessionId=$2
    fi

    if [ $SessionId = "SINGLE_ENTRY" ] || [ $((16#$SessionId)) -eq "$SESSION_ID_DUMMY" ]; then
        # no session id, choose new
        NewSessionId=$(dsyNewSessionId) || \
            return $DSY_ERR_LOC #error 
    else
        NewSessionId=$SessionId
    fi

    #prepare new sysup file
    dsyPrepareNewFileDisable $Name $SessionId $NewSessionId || \
        return $DSY_ERR_LOC #error 

    #disable sysup actions on rem side
    dsyDisableRem $Name $NewSessionId || \
        return $DSY_ERR_REM #error 

    #disable sysup actions on loc side
    mv ${SYSUP_FILE}.tmp $SYSUP_FILE
    if [ $? -ne $DSY_OK ]; then #error 
        dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in mv" 
        dsyEnableRem $Name $SessionId 
        return $DSY_ERR_LOC 
    fi

    echo $NewSessionId

    return $DSY_OK
}


##################################################
# Enables sysup actions on local node.
#
# @param Name         name of the caller of the script
# @param SessionId    session id
# @return Result      OK/ERROR                     
##################################################
function dsyEnableLoc
{
    Name=$1
    SessionId=$2

    if [ $SessionId = "ALL" ]; then
        #remove file
        rm  $SYSUP_FILE 2>/dev/null
        dsyLog "OK: file removed - sysup actions enabled"
        return $DSY_OK
    fi

    if [ $SessionId != "USER_ALL" ] && [ $((16#$SessionId)) -eq "$SESSION_ID_DUMMY" ]; then # no session id, error 
        dsyLogError $LINENO $DSY_ERR_REM "ERROR: Dummy Session ID" 
        return $DSY_ERR_LOC 
    fi

    #prepare new sysup file
    dsyPrepareNewFileEnable $Name $SessionId || \
        return $DSY_ERR_LOC #error 

    #enable sysup actions on loc side
    if [ -s ${SYSUP_FILE}.tmp ]; then  #file is not empty
        mv ${SYSUP_FILE}.tmp $SYSUP_FILE || \
            { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in mv" ; return $DSY_ERR_LOC; } #error 
    else #file is empty, remove it
        rm  $SYSUP_FILE 2>/dev/null
        dsyLog "OK: file removed - sysup actions enabled"
    fi

    return $DSY_OK
}


##################################################
# Enables sysup actions on remote node.
#
# @param Name         name of the caller of the script
# @param SessionId    session id
# @return Result      OK/ERROR                     
##################################################
function dsyEnableRem
{
    Name=$1
    SessionId=$2

    if [ $SessionId != "ALL" ] && [ $SessionId != "USER_ALL" ] && [ $((16#$SessionId)) -eq "$SESSION_ID_DUMMY" ]; then # no session id, error 
        dsyLogError $LINENO $DSY_ERR_REM "ERROR: Dummy Session ID" 
        return $DSY_ERR_REM 
    fi

    #call script on remote side
    RemSrv=$(dsyGetRemIp) || return $DSY_ERR_REM #read IP and return ERROR if not successfull
    SessionIdNew=`ssh -o ConnectTimeout=$SSH_TIMEOUT ${USER}@${RemSrv} "${MyScriptPath} enable $Name $SessionId" 2>/dev/null` || \
        { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Can not connect or error on remote server $RemSrv (check log on remote side)" ; return $DSY_ERR_REM; } #error 


    if [ "$SessionId" != "$SessionIdNew" ]; then #error
        dsyLogError $LINENO $DSY_ERR_REM "ERROR: Wrong SessionId got from remote" 
        return $DSY_ERR_REM
    fi

    return $DSY_OK
}


##################################################
# Enables sysup actions on both nodes.
#
# @param Name         name of the caller of the script
# @param SessionId    session id
# @return Result      OK/ERROR                     
##################################################
function dsyEnableHsb
{
    Name=$1
    SessionId=$2

    if [ $SessionId = "ALL" ]; then
        dsyEnableRem $Name $SessionId || \
            return $DSY_ERR_REM #error 
        #remove file
        rm  $SYSUP_FILE 2>/dev/null
        dsyLog "OK: file removed - sysup actions enabled"
        return $DSY_OK
    fi


    if [ $SessionId != "USER_ALL" ] && [ $((16#$SessionId)) -eq "$SESSION_ID_DUMMY" ]; then # no session id, error 
        dsyLogError $LINENO $DSY_ERR_REM "ERROR: Dummy Session ID" 
        return $DSY_ERR_REM 
    fi

    #prepare new sysup file
    dsyPrepareNewFileEnable $Name $SessionId || \
        return $DSY_ERR_LOC #error 

    #enable sysup actions on rem sid
    dsyEnableRem $Name $SessionId || \
        return $DSY_ERR_REM #error 

    #enable sysup actions on loc side
    if [ -s ${SYSUP_FILE}.tmp ]; then  #file is not empty
        mv ${SYSUP_FILE}.tmp $SYSUP_FILE || \
            { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in mv" ; return $DSY_ERR_LOC; } #error 
    else #file is empty, remove it
        rm  $SYSUP_FILE 2>/dev/null
        dsyLog "OK: file removed - sysup actions enabled"
    fi

    return $DSY_OK
}


##################################################
# Removes records in a file, which are there for a 
# long time (probably somebody forgets to remove them).
#
# @param none
# @return Result      OK/ERROR                     
##################################################
function dsyCheckFile
{
    if [ ! -e ${SYSUP_FILE} ]; then #check if file exists
        echo "0" 
        return $DSY_OK
    fi

    TimeNow=` date +"%s" ` || \
        { dsyLogError $LINENO $DSY_ERR_LOC "ERROR: Error in date" ; echo "0"; return $DSY_ERR_LOC; } #error 

    # check each record
    NrOfRemoved=0
    while read -r Name SessionId Time Rest
    do
        if [ $((Time+RECORD_MAX_TIMEOUT)) -lt "$TimeNow" ] ; then # record too long 
            dsyLog "ERROR: Record ($Name $SessionId $Time $Rest) has been in file for long time (more then ${RECORD_MAX_TIMEOUT}s) - removing it!"
            dsyEnableLoc $Name $SessionId && NrOfRemoved=$((NrOfRemoved+1))
        fi
    done < ${SYSUP_FILE}
    

    echo $NrOfRemoved
    return $DSY_OK
}

##################################################
# Checks status of sysup (enabled/disabled) and prints it
#
# @param none
# @return Result      OK/ERROR                     
##################################################
function dsyStatus
{
    if [ -e ${SYSUP_FILE} ]; then
        echo -n "SYSUP ACTIONS DISABLED - nr. of entries="
        wc -l $SYSUP_FILE
    else
        echo "SYSUP ACTIONS ENABLED"
    fi

    return $DSY_OK
}


##################################################
# Main function - executes requested operaton.
#
# @param Command      requested operation (disable/disable-hsb/enable/enable-hsb/status/check/help)
# @param MyName       name of the caller of the script
# @param MySessionId  session id (SESSION_ID_DUMMY if unknown)
# @return SessionId   session id of current session
# @return Result      OK/ERROR                     
##################################################

#check input params
if [ $# -lt 3 ] && [ $# -ge 1 ] && [ $1 != "check" ] && [ $1 != "status" ] && [ $1 != "help" ]; then
    echo "ERROR: Wrong input params!"
    echo
    dsyUsage
    exit $DSY_ERR_LOC
fi
Command=$1      #command (disable/disable-hsb/enable/enable-hsb/status)
MyName=$2       #name of the caller
MySessionId=$3  #session id

rm ${SYSUP_FILE}.tmp* 2>/dev/null

#create script name with path - used for ssh to rem side
MyScriptName=`basename "$0"`
if [ $(dirname $0) = '.' ]; then
    MyScriptDir=$(pwd)
else
    MyScriptDir=$(dirname $0)
fi
MyScriptPath=${MyScriptDir}"/"${MyScriptName}
#for testing
#MyScriptPath=/users/rdsa/kpog/MyPrograms/SYSUP_SCRIPT/sysup-disable.sh # for testing only

#create needed directories
mkdir -p $(dirname $SYSUP_FILE)

if [ $Command != "check" ]; then
    dsyLog "OK: executing $0 ${*}"
fi
#set -x
#lock script
dsyLock $MyScriptName
if [ $? -ne $DSY_OK ]; then #script locked
    dsyLogError $LINENO $DSY_ERR_LOC "WARNING: Script locked, other process is executing it"
    echo "0"
    exit $DSY_ERR_LOC 
fi

#execute command
case $Command in
    "disable")
        MySessionId=$(dsyDisableLoc $MyName $MySessionId)
        FuncResult=$?
        ;;
    "disable-hsb")
        SysArch=$(dsyGetSysArch)
        if [ $SysArch = "HSB" ]; then
            MySessionId=$(dsyDisableHsb $MyName $MySessionId)
        else
            MySessionId=$(dsyDisableLoc $MyName $MySessionId)
        fi
        FuncResult=$?
        ;;
    "enable")
        dsyEnableLoc $MyName $MySessionId
        FuncResult=$?
        ;;
    "enable-hsb")
        SysArch=$(dsyGetSysArch)
        if [ $SysArch = "HSB" ]; then
            dsyEnableHsb $MyName $MySessionId
        else
            dsyEnableLoc $MyName $MySessionId
        fi
        FuncResult=$?
        ;;
    "check")
        NrOfRemoved=$(dsyCheckFile) && \
            { if [ $NrOfRemoved -ne 0 ]; then dsyLog "OK: successfully executed $0 ${*} NrOfRemovedRecords=$NrOfRemoved"; echo $NrOfRemoved; fi; rm ${SYSUP_FILE}.tmp* 2>/dev/null; exit $DSY_OK; } || # OK \
            { dsyLog "OK: error while executing $0 ${*}"; echo "ERROR"; rm ${SYSUP_FILE}.tmp* 2>/dev/null; exit $DSY_ERR_LOC; } #error
        ;;
    "status")
        dsyStatus
        exit $DSY_OK
        ;;
    "help")
        echo
        dsyUsage
        echo
        exit $DSY_OK
        ;;
    *)
        echo
        echo "ERROR: Wrong input command!"
        dsyUsage
        echo
        exit $DSY_OK
        ;;
esac

rm ${SYSUP_FILE}.tmp* 2>/dev/null

if [ $FuncResult -ne $DSY_OK ]; then
   echo "0"
   exit $FuncResult
fi

#success
dsyLog "OK: successfully executed $0 ${*} NewSessionId=$MySessionId"
echo $MySessionId
exit $DSY_OK


